var Manager = function()
{
	var manager = {};
	
	var debug = 0;
	
	manager.initPage = function()
	{
		var url = '../Admin.servlet?ID=' + App.getSessionId();
		url += '&cmd=GetIndexInfo';
		Wizard.showLoadingAnimation('Loading index information...');
		Wizard.asyncJsonRequest(url, manager.indexInfoLoaded);
	}

	manager.refreshPage = function()
	{
		var url = '../Admin.servlet?ID=' + App.getSessionId();
		url += '&cmd=GetIndexInfo';
		Wizard.asyncJsonRequest(url, manager.indexInfoLoaded);
	}
	
	manager.indexInfoLoaded = function(response)
	{
		var html = '';
		var indexes = response.indexes;
		
		var autoRefresh = false;
		
		for (var idxNo = 0; idxNo < indexes.length; idxNo++)
		{
			var idx = indexes[idxNo];
			var queryCache = idx.queryCache;
			var itemList = idx.itemList;
			var progress = idx.progress;
			var allowActions = (idx.status == 'IDLE' || idx.status == 'DISABLED') && idx.nextAction == 'DEFAULT';
			if (!allowActions) autoRefresh = true;
			
			html += '<div class="index-div">';
			html += '<div class="index-name bg-filled-100">'+Strings.encodeTags(idx.name) + '</div>';
			html += '<div>Path</div><div>'+Strings.encodeTags(idx.path)+'</div>';
			html += '<div>';
			if (allowActions)
			{
				html += '<span class="link index-action" data-cmd="auto-update" data-auto-update="DELETE" data-confirm="Are you sure? This will remove everything from the index!" data-index-id="'+idx.id+'" title="Delete this index and create a new empty index">Delete...</span>';
			}
			else if (idx.status == 'IDLE' && idx.nextAction == 'DELETE')
			{
				html += '<span class="next-action">Delete (waiting)</span>';
			}
			html += '</div>';
			html += '<div>Size on disk</div><div>'+idx.sizeOnDiskFormatted+'</div><div></div>';
			html += '<div>Total variants</div><div>'+VarSearch.niceCount(idx.numVariants)+'</div><div></div>';
			html += '<div>Indexed raw bioassays</div><div>'+VarSearch.niceCount(idx.numRawBioAssays)+'</div>';
			html += '<div>';
			if (allowActions && idx.numRawBioAssays > 0)
			{
				html += '<span class="link index-action" data-cmd="auto-update" data-auto-update="FULL_REBUILD" data-index-id="'+idx.id+'" title="Re-index all rawbioassays">Rebuild index</span>';
			}
			else if (idx.status == 'IDLE' && idx.nextAction == 'FULL_REBUILD')
			{
				html += '<span class="next-action">Rebuild index (waiting)</span>';
			}
			html += '</div>';
			
			html += '<div>Deleted raw bioassays</div>';
			html += '<div>'+VarSearch.niceCount(idx.nonExistingRawBioAssays)+'</div>';
			html += '<div>';
			if (allowActions && idx.nonExistingRawBioAssays > 0)
			{
				html += '<span class="link index-action" data-cmd="auto-update" data-auto-update="REMOVE_NON_EXISTING" data-index-id="'+idx.id+'" title="Remove all non-existing raw bioassays from the index">Remove from index</span>';
			}
			else if (idx.status == 'IDLE' && idx.nextAction == 'REMOVE_NON_EXISTING')
			{
				html += '<span class="next-action">Remove from index (waiting)</span>';
			}
			html += '</div>';
			
			if (queryCache)
			{
				html += '<div>Cached query results</div>';
				html += '<div>'+VarSearch.niceCount(queryCache.size)+'</div>';
				html += '<div>';
				if (queryCache.size > 0)
				{
					html += '<span class="link index-action" data-cmd="clean-cache" data-index-id="'+idx.id+'" title="Clean all results from the query cache">Clean cache</span>';
				}
				html += '</div>';
			}
			
			if (itemList)
			{
				// Item list
				html += '<div>Item list</div>';
				html += '<div><span class="link linked-item" data-item-type="ITEMLIST" data-item-id="'+itemList.id+'">'+Strings.encodeTags(itemList.name)+'</span></div>';
				html += '<div></div>';
				
				// AutoProcessing
				html += '<div>Auto-processing</div>';
				if (itemList.autoProcessing == 'Disable')
				{
					html += '<div>Disabled</div>';
					html += '<div><span class="link index-action" data-cmd="change-ap" data-list-id="'+itemList.id+'" data-auto-processing="" title="Enable auto-processing">Enable</span></div>';
				}
				else
				{
					html += '<div>Enabled</div>';
					html += '<div><span class="link index-action" data-cmd="change-ap" data-list-id="'+itemList.id+'" data-auto-processing="Disable" title="Disable auto-processing">Disable</span></div>';
				}
				
				// Items in queue
				html += '<div>Items in queue</div>';
				html += '<div>'+VarSearch.niceCount(itemList.size)+'</div>';
				html += '<div>';
				if (allowActions && itemList.size > 0)
				{
					html += '<span class="link index-action" data-cmd="auto-update" data-auto-update="ADD_TO_INDEX" data-index-id="'+idx.id+'" title="Add all items to the index">Add to index</span>';
				}
				else if (idx.status == 'IDLE' && idx.nextAction == 'ADD_TO_INDEX')
				{
					html += '<span class="next-action">Add to index (waiting)</span>';
				}
				html += '</div>';
			}
			
			html += '<div>Status</div>';
			html += '<div>'+Strings.encodeTags(idx.status);
			if (idx.status == 'IDLE' && idx.nextAction != 'DEFAULT') 
			{
				autoRefresh = true;
				html += '<span class="next-action">' + Strings.encodeTags(idx.nextAction) + '</span>';
			}
			html += '</div><div></div>';
			
			if (progress)
			{
				autoRefresh = true;
				html += '<div>'+Strings.encodeTags(progress.message)+'</div>';
				html += '<div><div class="wizard-progress progress-bar" value="'+progress.percent+'%"><span class="progress-bar-done bg-filled-100 topborder bottomborder leftborder" style="width: '+progress.percent+'%;"></span><span class="progress-bar-remain topborder bottomborder rightborder"style="width: '+(100-progress.percent)+'%;"></span></div></div><div></div>';
			}
			
			html += '</div>'
		}
		
		Doc.element('indexList').innerHTML = html;
		
		
		var linkedItems = document.getElementsByClassName('linked-item');
		for (var itemNo = 0; itemNo < linkedItems.length; itemNo++)
		{
			Events.addEventHandler(linkedItems[itemNo], 'click', Items.itemOnClick);
		}
		
		var indexAction = document.getElementsByClassName('index-action');
		for (var itemNo = 0; itemNo < indexAction.length; itemNo++)
		{
			var action = indexAction[itemNo];
			if (!Doc.isDisabled(action))
			{
				Events.addEventHandler(action, 'click', manager.indexAction);
			}
		}
		
		if (autoRefresh) setTimeout(manager.refreshPage, 1500);
	}

	manager.indexAction = function(event)
	{
		var cmd = Data.get(event.currentTarget, 'cmd');
		var msg = Data.get(event.currentTarget, 'confirm');
		if (msg && !confirm(msg)) return;
		
		if (cmd == 'change-ap') manager.changeAutoProcessing(event);
		if (cmd == 'clean-cache') manager.cleanCache(event);
		if (cmd == 'auto-update') manager.autoUpdate(event);
	}

	manager.changeAutoProcessing = function(event)
	{
		var element = event.currentTarget;

		var submitInfo = {};
		submitInfo.listId = Data.int(element, 'list-id');
		submitInfo.autoProcessing = Data.get(element, 'auto-processing');

		var url = '../Admin.servlet?ID=' + App.getSessionId();
		url += '&cmd=ChangeAutoProcessing';
		Wizard.asyncJsonRequest(url, manager.initPage, 'POST', JSON.stringify(submitInfo));
		Doc.element('indexList').innerHTML = '';
	}
	
	manager.cleanCache = function(event)
	{
		var element = event.currentTarget;

		var submitInfo = {};
		submitInfo.indexId = Data.get(element, 'index-id');

		var url = '../Admin.servlet?ID=' + App.getSessionId();
		url += '&cmd=CleanCache';
		Wizard.asyncJsonRequest(url, manager.initPage, 'POST', JSON.stringify(submitInfo));
		Doc.element('indexList').innerHTML = '';
	}

	manager.autoUpdate = function(event)
	{
		var element = event.currentTarget;

		var submitInfo = {};
		submitInfo.indexId = Data.get(element, 'index-id');
		submitInfo.autoUpdate = Data.get(element, 'auto-update');

		var url = '../Admin.servlet?ID=' + App.getSessionId();
		url += '&cmd=SetNextAutoUpdate';
		Wizard.asyncJsonRequest(url, manager.initPage, 'POST', JSON.stringify(submitInfo));
		Doc.element('indexList').innerHTML = '';
	}

	return manager;
}();

Doc.onLoad(Manager.initPage);
